/*
 * Uloha1Test.java
 * JUnit based test
 *
 */

package uloha1;

import java.io.IOException;
import java.net.URL;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 *
 * @author Petr Adamek
 */
public class Uloha1Test extends TestCase {
    
    public Uloha1Test(String testName) {
        super(testName);
    }
    
    Uloha1 uloha1;
    XPath xpath;
    
    protected void setUp() throws Exception {
        URL url = Uloha1Test.class.getResource("people.xml");
        if (url == null) {
            throw new IOException("Nelze nalezt testovaci XML soubor");
        }
        uloha1 = Uloha1.newInstance(url);
        
        xpath = XPathFactory.newInstance().newXPath();
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(Uloha1Test.class);
        return suite;
    }

    /**
     * Test of adjustSalary method, of class uloha1.Uloha1.
     */
    public void testAdjustSalary() throws XPathExpressionException {
        double[] salaries = { 1000, 5000, 10000, 15000, 20000, 100000 };
        for (double salary : salaries) {

            NodeList peopleWithGreaterSalary = (NodeList) xpath.evaluate(
                    "//person[salary > " + salary + "]",
                    uloha1.getDocument(), XPathConstants.NODESET);
            
            uloha1.adjustSalary(salary);
            NodeList peopleWithLessSalary = (NodeList) xpath.evaluate(
                    "//person[salary < " + salary + "]",
                    uloha1.getDocument(), XPathConstants.NODESET);

            NodeList peopleWithGreaterSalaryAfter = (NodeList) xpath.evaluate(
                    "//person[salary > " + salary + "]",
                    uloha1.getDocument(), XPathConstants.NODESET);
            
            assertTrue("Po volani metody adjustSalary s parametrem salary = " + 
                    salary + " stale existuji osoby s mensim platem",
                    peopleWithLessSalary.getLength() == 0);
            
            assertEquals("Po volani metody adjustSalary s parametrem salary = " + 
                    salary + " se zmenil i pocet osob s vetsim platem",
                    peopleWithGreaterSalary.getLength(),
                    peopleWithGreaterSalaryAfter.getLength());
            
        }
    }

    /**
     * Test of changeName method, of class uloha1.Uloha1.
     */
    public void testChangeName() throws XPathExpressionException {
        uloha1.changeName("1","Jindra");
        assertEquals("Jindra",xpath.evaluate(
            "//person[@id=1]/name",uloha1.getDocument()));

        uloha1.changeName("1","Pavel");
        assertEquals("Pavel",xpath.evaluate(
            "//person[@id=1]/name",uloha1.getDocument()));
        
        uloha1.changeName("2","Petr");
        assertEquals("Petr",xpath.evaluate(
            "//person[@id=2]/name",uloha1.getDocument()));
    }

    /**
     * Test of getWageCosts method, of class uloha1.Uloha1.
     */
    public void testGetWageCosts() throws XPathExpressionException {
        assertEquals(getWageCosts(uloha1.getDocument()),uloha1.getWageCosts());
        uloha1.adjustSalary(14000);
        assertEquals(getWageCosts(uloha1.getDocument()),uloha1.getWageCosts());
    }

    private double getWageCosts(Document doc) throws XPathExpressionException {
        return (Double) xpath.evaluate(
            "sum(//salary)",uloha1.getDocument(), XPathConstants.NUMBER);
    }
}
